@@ -3,7 +3,7 @@ import traceback
 
 from fastapi import FastAPI, Header, HTTPException, Query, Request, status
 from fastapi.middleware.cors import CORSMiddleware
-from fastapi.responses import HTMLResponse, RedirectResponse, Response, UJSONResponse
+from fastapi.responses import HTMLResponse, JSONResponse, RedirectResponse, Response
 from pydantic import BaseModel, Field
 from slowapi import Limiter, _rate_limit_exceeded_handler
 from slowapi.errors import RateLimitExceeded
@@ -165,7 +165,7 @@ async def bot() -> Response:
 
     Returns a message when accessed by suspicious user agents.
     """
-    return UJSONResponse({'bot': 'These are not the DNS records you are looking for'})
+    return JSONResponse({'bot': 'These are not the DNS records you are looking for'})
 
 
 class CapabilitiesResponse(BaseModel):
@@ -190,7 +190,7 @@ async def get_capabilities(request: Request) -> Response:
     """
     try:
         capabilities = [
-            'general_enum - General DNS enumeration with multiple sources',
+            'general_enum - General DNS enumeration with multiple sources (supports do_shodan, shodan_active, X-Shodan-Api-Key header)',
             'brute_domain - Domain brute forcing',
             'brute_srv - SRV record brute forcing',
             'brute_tlds - TLD brute forcing',
@@ -204,12 +204,12 @@ async def get_capabilities(request: Request) -> Response:
             'cache_snoop - DNS cache snooping',
             'nxdomain_hijack - NXDOMAIN hijacking check',
         ]
-        return UJSONResponse({'capabilities': capabilities})
+        return JSONResponse({'capabilities': capabilities})
     except Exception as e:
         error_traceback = traceback.format_exc()
         print(f'Error in get_capabilities endpoint: {e!s}\n{error_traceback}')
 
-        return UJSONResponse(
+        return JSONResponse(
             {
                 'detail': f'An error occurred while retrieving capabilities: {e!s}',
                 'error_type': type(e).__name__,
@@ -231,7 +231,12 @@ async def get_capabilities(request: Request) -> Response:
 @limiter.limit(API_RATE_LIMIT)
 async def general_enumeration(
     request: Request,
-    user_agent: str = Header(None),
+    user_agent: str | None = Header(None),
+    shodan_api_key_header: str | None = Header(
+        None,
+        alias='X-Shodan-Api-Key',
+        description='Optional Shodan API key header for Shodan-backed netblock expansion',
+    ),
     domain: str = Query(..., description='Domain to enumerate'),
     do_axfr: bool = Query(False, description='Perform zone transfer'),
     do_bing: bool = Query(False, description='Use Bing search'),
@@ -239,6 +244,8 @@ async def general_enumeration(
     do_spf: bool = Query(False, description='Check SPF records'),
     do_whois: bool = Query(False, description='Perform WHOIS lookup'),
     do_crt: bool = Query(False, description='Check certificate transparency'),
+    do_shodan: bool = Query(False, description='Use Shodan to expand SPF/Whois-discovered netblocks'),
+    shodan_active: bool = Query(False, description='Actively validate Shodan-discovered hosts using DNS resolution'),
     zw: bool = Query(False, description='Perform zone walking'),
     request_timeout: int = Query(3, description='Request timeout in seconds'),
     thread_num: int = Query(10, description='Number of threads to use'),
@@ -263,6 +270,8 @@ async def general_enumeration(
         # Create DNS resolver
         res = DnsHelper(domain, recursion_desired=recursion_desired)
 
+        shodan_api_key = shodan_api_key_header or os.getenv('SHODAN_API_KEY')
+
         # Perform general enumeration
         results = general_enum(
             res=res,
@@ -273,6 +282,9 @@ async def general_enumeration(
             do_spf=do_spf,
             do_whois=do_whois,
             do_crt=do_crt,
+            do_shodan=do_shodan,
+            shodan_api_key=shodan_api_key,
+            shodan_active=shodan_active,
             zw=zw,
             request_timeout=request_timeout,
             thread_num=thread_num,
@@ -297,7 +309,7 @@ async def general_enumeration(
                     if name and name != domain:
                         subdomains.append(name)
 
-        return UJSONResponse(
+        return JSONResponse(
             {
                 'domain': domain,
                 'records': [record.model_dump() for record in records],
@@ -312,7 +324,7 @@ async def general_enumeration(
         error_traceback = traceback.format_exc()
         print(f'Error in general_enumeration endpoint: {e!s}\n{error_traceback}')
 
-        return UJSONResponse(
+        return JSONResponse(
             {
                 'detail': f'An error occurred while processing your request: {e!s}',
                 'error_type': type(e).__name__,
@@ -395,7 +407,7 @@ async def brute_force_domain(
                     if name:
                         subdomains.append(name)
 
-        return UJSONResponse(
+        return JSONResponse(
             {'domain': domain, 'subdomains': list(set(subdomains)), 'records': [record.model_dump() for record in records]}
         )
 
@@ -405,7 +417,7 @@ async def brute_force_domain(
         error_traceback = traceback.format_exc()
         print(f'Error in brute_force_domain endpoint: {e!s}\n{error_traceback}')
 
-        return UJSONResponse(
+        return JSONResponse(
             {
                 'detail': f'An error occurred while processing your request: {e!s}',
                 'error_type': type(e).__name__,
@@ -473,7 +485,7 @@ async def brute_force_reverse(
 
                     records.append(DnsRecord(name=name, type='PTR', address=address))
 
-        return UJSONResponse({'ip_range': ip_range, 'records': [record.model_dump() for record in records]})
+        return JSONResponse({'ip_range': ip_range, 'records': [record.model_dump() for record in records]})
 
     except HTTPException as e:
         raise e
@@ -481,7 +493,7 @@ async def brute_force_reverse(
         error_traceback = traceback.format_exc()
         print(f'Error in brute_force_reverse endpoint: {e!s}\n{error_traceback}')
 
-        return UJSONResponse(
+        return JSONResponse(
             {
                 'detail': f'An error occurred while processing your request: {e!s}',
                 'error_type': type(e).__name__,
@@ -530,7 +542,7 @@ async def wildcard_check(
         if not wildcard_ips:
             wildcard_ips = []
 
-        return UJSONResponse({'domain': domain, 'wildcard_enabled': wildcard_enabled, 'wildcard_ips': wildcard_ips})
+        return JSONResponse({'domain': domain, 'wildcard_enabled': wildcard_enabled, 'wildcard_ips': wildcard_ips})
 
     except HTTPException as e:
         raise e
@@ -538,7 +550,7 @@ async def wildcard_check(
         error_traceback = traceback.format_exc()
         print(f'Error in wildcard_check endpoint: {e!s}\n{error_traceback}')
 
-        return UJSONResponse(
+        return JSONResponse(
             {
                 'detail': f'An error occurred while processing your request: {e!s}',
                 'error_type': type(e).__name__,
@@ -604,7 +616,7 @@ async def brute_force_srv(
 
                     srv_records.append(DnsRecord(name=name, type=record_type, address=address, target=target, port=port))
 
-        return UJSONResponse({'domain': domain, 'srv_records': [record.model_dump() for record in srv_records]})
+        return JSONResponse({'domain': domain, 'srv_records': [record.model_dump() for record in srv_records]})
 
     except HTTPException as e:
         raise e
@@ -612,7 +624,7 @@ async def brute_force_srv(
         error_traceback = traceback.format_exc()
         print(f'Error in brute_force_srv endpoint: {e!s}\n{error_traceback}')
 
-        return UJSONResponse(
+        return JSONResponse(
             {
                 'detail': f'An error occurred while processing your request: {e!s}',
                 'error_type': type(e).__name__,
@@ -676,7 +688,7 @@ async def brute_force_tlds(
 
                     tld_records.append(DnsRecord(name=name, type=record_type, address=address))
 
-        return UJSONResponse({'domain': domain, 'tld_records': [record.model_dump() for record in tld_records]})
+        return JSONResponse({'domain': domain, 'tld_records': [record.model_dump() for record in tld_records]})
 
     except HTTPException as e:
         raise e
@@ -684,7 +696,7 @@ async def brute_force_tlds(
         error_traceback = traceback.format_exc()
         print(f'Error in brute_force_tlds endpoint: {e!s}\n{error_traceback}')
 
-        return UJSONResponse(
+        return JSONResponse(
             {
                 'detail': f'An error occurred while processing your request: {e!s}',
                 'error_type': type(e).__name__,
@@ -751,7 +763,7 @@ async def axfr_test(
 
                     records.append(DnsRecord(name=name, type=record_type, address=address, target=target))
 
-        return UJSONResponse(
+        return JSONResponse(
             {
                 'domain': domain,
                 'zone_transfer_successful': zone_transfer_successful,
@@ -765,7 +777,7 @@ async def axfr_test(
         error_traceback = traceback.format_exc()
         print(f'Error in axfr_test endpoint: {e!s}\n{error_traceback}')
 
-        return UJSONResponse(
+        return JSONResponse(
             {
                 'detail': f'An error occurred while processing your request: {e!s}',
                 'error_type': type(e).__name__,
@@ -824,7 +836,7 @@ async def caa_records(
 
                     caa_records_list.append(DnsRecord(name=name, type=record_type, address=value))
 
-        return UJSONResponse({'domain': domain, 'caa_records': [record.model_dump() for record in caa_records_list]})
+        return JSONResponse({'domain': domain, 'caa_records': [record.model_dump() for record in caa_records_list]})
 
     except HTTPException as e:
         raise e
@@ -832,7 +844,7 @@ async def caa_records(
         error_traceback = traceback.format_exc()
         print(f'Error in caa_records endpoint: {e!s}\n{error_traceback}')
 
-        return UJSONResponse(
+        return JSONResponse(
             {
                 'detail': f'An error occurred while processing your request: {e!s}',
                 'error_type': type(e).__name__,
@@ -909,7 +921,7 @@ async def cache_snoop(
 
                     cached_records.append(DnsRecord(name=name, type=record_type, address=address))
 
-        return UJSONResponse({'nameserver': nameserver, 'cached_records': [record.model_dump() for record in cached_records]})
+        return JSONResponse({'nameserver': nameserver, 'cached_records': [record.model_dump() for record in cached_records]})
 
     except HTTPException as e:
         raise e
@@ -917,7 +929,7 @@ async def cache_snoop(
         error_traceback = traceback.format_exc()
         print(f'Error in cache_snoop endpoint: {e!s}\n{error_traceback}')
 
-        return UJSONResponse(
+        return JSONResponse(
             {
                 'detail': f'An error occurred while processing your request: {e!s}',
                 'error_type': type(e).__name__,
@@ -981,7 +993,7 @@ async def zone_walk(
 
                     zone_walk_records.append(DnsRecord(name=name, type=record_type, address=address, target=target))
 
-        return UJSONResponse({'domain': domain, 'zone_walk_records': [record.model_dump() for record in zone_walk_records]})
+        return JSONResponse({'domain': domain, 'zone_walk_records': [record.model_dump() for record in zone_walk_records]})
 
     except HTTPException as e:
         raise e
@@ -989,7 +1001,7 @@ async def zone_walk(
         error_traceback = traceback.format_exc()
         print(f'Error in zone_walk endpoint: {e!s}\n{error_traceback}')
 
-        return UJSONResponse(
+        return JSONResponse(
             {
                 'detail': f'An error occurred while processing your request: {e!s}',
                 'error_type': type(e).__name__,
@@ -1045,7 +1057,7 @@ async def bind_version(
         version_detected = bool(version_info)
         bind_version = version_info if version_info else 'Version not detected'
 
-        return UJSONResponse({'nameserver': nameserver, 'bind_version': bind_version, 'version_detected': version_detected})
+        return JSONResponse({'nameserver': nameserver, 'bind_version': bind_version, 'version_detected': version_detected})
 
     except HTTPException as e:
         raise e
@@ -1053,7 +1065,7 @@ async def bind_version(
         error_traceback = traceback.format_exc()
         print(f'Error in bind_version endpoint: {e!s}\n{error_traceback}')
 
-        return UJSONResponse(
+        return JSONResponse(
             {
                 'detail': f'An error occurred while processing your request: {e!s}',
                 'error_type': type(e).__name__,
@@ -1109,7 +1121,7 @@ async def recursive_check(
         recursive_enabled = bool(recursion_result)
         test_result = recursion_result if recursion_result else 'Recursion not enabled or test failed'
 
-        return UJSONResponse({'nameserver': nameserver, 'recursive_enabled': recursive_enabled, 'test_result': test_result})
+        return JSONResponse({'nameserver': nameserver, 'recursive_enabled': recursive_enabled, 'test_result': test_result})
 
     except HTTPException as e:
         raise e
@@ -1117,7 +1129,7 @@ async def recursive_check(
         error_traceback = traceback.format_exc()
         print(f'Error in recursive_check endpoint: {e!s}\n{error_traceback}')
 
-        return UJSONResponse(
+        return JSONResponse(
             {
                 'detail': f'An error occurred while processing your request: {e!s}',
                 'error_type': type(e).__name__,
@@ -1169,7 +1181,7 @@ async def nxdomain_hijack(
         hijack_detected = bool(hijack_result)
         hijack_details = hijack_result if hijack_result else 'No NXDOMAIN hijacking detected'
 
-        return UJSONResponse({'nameserver': nameserver, 'hijack_detected': hijack_detected, 'hijack_details': hijack_details})
+        return JSONResponse({'nameserver': nameserver, 'hijack_detected': hijack_detected, 'hijack_details': hijack_details})
 
     except HTTPException as e:
         raise e
@@ -1177,7 +1189,7 @@ async def nxdomain_hijack(
         error_traceback = traceback.format_exc()
         print(f'Error in nxdomain_hijack endpoint: {e!s}\n{error_traceback}')
 
-        return UJSONResponse(
+        return JSONResponse(
             {
                 'detail': f'An error occurred while processing your request: {e!s}',
                 'error_type': type(e).__name__,
